@@ -39,10 +39,12 @@
 
 #include "emu.h"
 #include "cococart.h"
+#include "formats/rpk.h"
 
 #include "coco_dcmodem.h"
 #include "coco_fdc.h"
 #include "coco_gmc.h"
+#include "coco_ide.h"
 #include "coco_max.h"
 #include "coco_midi.h"
 #include "coco_multi.h"
@@ -71,7 +73,6 @@
     PARAMETERS
 ***************************************************************************/
 
-//#define LOG_GENERAL   (1U << 0) //defined in logmacro.h already
 #define LOG_CART (1U << 1) // shows cart line changes
 #define LOG_NMI  (1U << 2) // shows switch changes
 #define LOG_HALT (1U << 3) // shows switch changes
@@ -88,11 +89,20 @@
     CONSTANTS
 ***************************************************************************/
 
-enum
+// definitions of RPK PCBs in layout.xml
+static const char *coco_rpk_pcbdefs[] =
 {
-	TIMER_CART,
-	TIMER_NMI,
-	TIMER_HALT
+	"standard",
+	"paged16k",
+	nullptr
+};
+
+
+// ...and their mappings to "default card slots"
+static const char *coco_rpk_cardslottypes[] =
+{
+	"pak",
+	"banked_16k"
 };
 
 
@@ -107,13 +117,15 @@ DEFINE_DEVICE_TYPE(COCOCART_SLOT, cococart_slot_device, "cococart_slot", "CoCo C
 //  LIVE DEVICE
 //**************************************************************************
 
+ALLOW_SAVE_TYPE(cococart_slot_device::line_value);
+
 //-------------------------------------------------
 //  cococart_slot_device - constructor
 //-------------------------------------------------
 cococart_slot_device::cococart_slot_device(const machine_config &mconfig, const char *tag, device_t *owner, u32 clock) :
 	device_t(mconfig, COCOCART_SLOT, tag, owner, clock),
 	device_single_card_slot_interface<device_cococart_interface>(mconfig, *this),
-	device_image_interface(mconfig, *this),
+	device_cartrom_image_interface(mconfig, *this),
 	m_cart_callback(*this),
 	m_nmi_callback(*this),
 	m_halt_callback(*this), m_cart(nullptr)
@@ -128,11 +140,11 @@ cococart_slot_device::cococart_slot_device(const machine_config &mconfig, const
 
 void cococart_slot_device::device_start()
 {
-	for(int i=0; i<TIMER_POOL; i++ )
+	for(int i=0; i < TIMER_POOL; i++ )
 	{
-		m_cart_line.timer[i]    = timer_alloc(TIMER_CART);
-		m_nmi_line.timer[i]     = timer_alloc(TIMER_NMI);
-		m_halt_line.timer[i]    = timer_alloc(TIMER_HALT);
+		m_cart_line.timer[i]    = timer_alloc(FUNC(cococart_slot_device::cart_line_timer_tick), this);
+		m_nmi_line.timer[i]     = timer_alloc(FUNC(cococart_slot_device::nmi_line_timer_tick), this);
+		m_halt_line.timer[i]    = timer_alloc(FUNC(cococart_slot_device::halt_line_timer_tick), this);
 	}
 
 	m_cart_line.timer_index     = 0;
@@ -140,7 +152,7 @@ void cococart_slot_device::device_start()
 	m_cart_line.value           = line_value::CLEAR;
 	m_cart_line.line            = 0;
 	m_cart_line.q_count         = 0;
-	m_cart_callback.resolve();
+	m_cart_callback.resolve_safe();
 	m_cart_line.callback = &m_cart_callback;
 
 	m_nmi_line.timer_index      = 0;
@@ -148,7 +160,7 @@ void cococart_slot_device::device_start()
 	m_nmi_line.value            = line_value::CLEAR;
 	m_nmi_line.line             = 0;
 	m_nmi_line.q_count          = 0;
-	m_nmi_callback.resolve();
+	m_nmi_callback.resolve_safe();
 	m_nmi_line.callback = &m_nmi_callback;
 
 	m_halt_line.timer_index     = 0;
@@ -156,36 +168,63 @@ void cococart_slot_device::device_start()
 	m_halt_line.value           = line_value::CLEAR;
 	m_halt_line.line            = 0;
 	m_halt_line.q_count         = 0;
-	m_halt_callback.resolve();
+	m_halt_callback.resolve_safe();
 	m_halt_line.callback = &m_halt_callback;
 
 	m_cart = get_card_device();
+
+	save_item(STRUCT_MEMBER(m_cart_line, timer_index));
+	save_item(STRUCT_MEMBER(m_cart_line, delay));
+	save_item(STRUCT_MEMBER(m_cart_line, value));
+	save_item(STRUCT_MEMBER(m_cart_line, line));
+	save_item(STRUCT_MEMBER(m_cart_line, q_count));
+
+	save_item(STRUCT_MEMBER(m_nmi_line, timer_index));
+	save_item(STRUCT_MEMBER(m_nmi_line, delay));
+	save_item(STRUCT_MEMBER(m_nmi_line, value));
+	save_item(STRUCT_MEMBER(m_nmi_line, line));
+	save_item(STRUCT_MEMBER(m_nmi_line, q_count));
+
+	save_item(STRUCT_MEMBER(m_halt_line, timer_index));
+	save_item(STRUCT_MEMBER(m_halt_line, delay));
+	save_item(STRUCT_MEMBER(m_halt_line, value));
+	save_item(STRUCT_MEMBER(m_halt_line, line));
+	save_item(STRUCT_MEMBER(m_halt_line, q_count));
 }
 
 
 
 //-------------------------------------------------
-//  device_timer - handle timer callbacks
+//  cart_line_timer_tick - update the output
+//  value for the cart's line to PIA1 CB1
 //-------------------------------------------------
 
-void cococart_slot_device::device_timer(emu_timer &timer, device_timer_id id, int param, void *ptr)
+TIMER_CALLBACK_MEMBER(cococart_slot_device::cart_line_timer_tick)
 {
-	switch(id)
-	{
-		case TIMER_CART:
-			set_line(line::CART, m_cart_line, (line_value) param);
-			break;
-
-		case TIMER_NMI:
-			set_line(line::NMI, m_nmi_line, (line_value) param);
-			break;
-
-		case TIMER_HALT:
-			set_line(line::HALT, m_halt_line, (line_value) param);
-			break;
-	}
+	set_line(line::CART, m_cart_line, (line_value) param);
 }
 
+//-------------------------------------------------
+//  nmi_line_timer_tick - update the output
+//  value sent to the CPU's NMI line
+//-------------------------------------------------
+
+TIMER_CALLBACK_MEMBER(cococart_slot_device::nmi_line_timer_tick)
+{
+	set_line(line::NMI, m_nmi_line, (line_value) param);
+}
+
+//-------------------------------------------------
+//  halt_line_timer_tick - update the output
+//  value sent to the CPU's HALT line
+//-------------------------------------------------
+
+TIMER_CALLBACK_MEMBER(cococart_slot_device::halt_line_timer_tick)
+{
+	set_line(line::HALT, m_halt_line, (line_value) param);
+}
+
+
 
 //-------------------------------------------------
 //  cts_read
@@ -306,9 +345,8 @@ void cococart_slot_device::set_line(line ln, coco_cartridge_line &line, cococart
 				break;
 		}
 
-		/* invoke the callback, if present */
-		if (!(*line.callback).isnull())
-			(*line.callback)(line.line);
+		/* invoke the callback */
+		(*line.callback)(line.line);
 	}
 }
 
@@ -474,11 +512,68 @@ void cococart_slot_device::set_cart_base_update(cococart_base_update_delegate up
 }
 
 
+//-------------------------------------------------
+//  read_coco_rpk
+//-------------------------------------------------
+
+static std::error_condition read_coco_rpk(std::unique_ptr<util::random_read> &&stream, rpk_file::ptr &result)
+{
+	// sanity checks
+	static_assert(std::size(coco_rpk_pcbdefs) - 1 == std::size(coco_rpk_cardslottypes));
+
+	// set up the RPK reader
+	rpk_reader reader(coco_rpk_pcbdefs, false);
+
+	// and read the RPK file
+	return reader.read(std::move(stream), result);
+}
+
+
+//-------------------------------------------------
+//  read_coco_rpk
+//-------------------------------------------------
+
+static std::error_condition read_coco_rpk(std::unique_ptr<util::random_read> &&stream, u8 *mem, offs_t cart_length, offs_t &actual_length)
+{
+	actual_length = 0;
+
+	// open the RPK
+	rpk_file::ptr file;
+	std::error_condition err = read_coco_rpk(std::move(stream), file);
+	if (err)
+		return err;
+
+	// for now, we are just going to load all sockets into the contiguous block of memory
+	// that cartridges use
+	offs_t pos = 0;
+	for (const rpk_socket &socket : file->sockets())
+	{
+		// only ROM supported for now; if we see anything else it should have been caught in the RPK code
+		assert(socket.type() == rpk_socket::socket_type::ROM);
+
+		// read all bytes
+		std::vector<uint8_t> contents;
+		err = socket.read_file(contents);
+		if (err)
+			return err;
+
+		// copy the bytes
+		offs_t size = (offs_t) std::min(contents.size(), (size_t)cart_length - pos);
+		memcpy(&mem[pos], &contents[0], size);
+		pos += size;
+	}
+
+	// we're done!
+	actual_length = pos;
+	return std::error_condition();
+}
+
+
 //-------------------------------------------------
 //  call_load
 //-------------------------------------------------
 
-image_init_result cococart_slot_device::call_load()
+std::pair<std::error_condition, std::string> cococart_slot_device::call_load()
 {
 	if (m_cart)
 	{
@@ -486,14 +581,26 @@ image_init_result cococart_slot_device::call_load()
 		u8 *base = cart_mem->base();
 		offs_t read_length, cart_length = cart_mem->bytes();
 
-		if (!loaded_through_softlist())
+		if (loaded_through_softlist())
 		{
-			read_length = fread(base, cart_length);
+			// loaded through softlist
+			read_length = get_software_region_length("rom");
+			memcpy(base, get_software_region("rom"), read_length);
+		}
+		else if (is_filetype("rpk"))
+		{
+			// RPK file
+			util::core_file::ptr proxy;
+			std::error_condition err = util::core_file::open_proxy(image_core_file(), proxy);
+			if (!err)
+				err = read_coco_rpk(std::move(proxy), base, cart_length, read_length);
+			if (err)
+				return std::make_pair(err, std::string());
 		}
 		else
 		{
-			read_length = get_software_region_length("rom");
-			memcpy(base, get_software_region("rom"), read_length);
+			// conventional ROM image
+			read_length = fread(base, cart_length);
 		}
 
 		while (read_length < cart_length)
@@ -503,7 +610,7 @@ image_init_result cococart_slot_device::call_load()
 			read_length += len;
 		}
 	}
-	return image_init_result::PASS;
+	return std::make_pair(std::error_condition(), std::string());
 }
 
 
@@ -513,9 +620,25 @@ image_init_result cococart_slot_device::call_load()
 
 std::string cococart_slot_device::get_default_card_software(get_default_card_software_hook &hook) const
 {
-	return software_get_default_slot("pak");
-}
+	// this is the default for anything not in an RPK file
+	int pcb_type = 0;
 
+	// is this an RPK?
+	if (hook.is_filetype("rpk"))
+	{
+		// RPK file
+		rpk_file::ptr file;
+		util::core_file::ptr proxy;
+		std::error_condition err = util::core_file::open_proxy(*hook.image_file(), proxy);
+		if (!err)
+			err = read_coco_rpk(std::move(proxy), file);
+		if (!err)
+			pcb_type = file->pcb_type();
+	}
+
+	// lookup the default slot
+	return software_get_default_slot(coco_rpk_cardslottypes[pcb_type]);
+}
 
 
 //**************************************************************************
@@ -532,7 +655,7 @@ template class device_finder<device_cococart_interface, true>;
 
 device_cococart_interface::device_cococart_interface(const machine_config &mconfig, device_t &device)
 	: device_interface(device, "cococart")
-	, m_owning_slot(nullptr)
+	, m_owning_slot(dynamic_cast<cococart_slot_device *>(device.owner()))
 	, m_host(nullptr)
 {
 }
@@ -553,7 +676,6 @@ device_cococart_interface::~device_cococart_interface()
 
 void device_cococart_interface::interface_config_complete()
 {
-	m_owning_slot = dynamic_cast<cococart_slot_device *>(device().owner());
 	m_host = m_owning_slot
 			? dynamic_cast<device_cococart_host_interface *>(m_owning_slot->owner())
 			: nullptr;
@@ -711,13 +833,14 @@ void coco_cart_add_basic_devices(device_slot_interface &device)
 	device.option_add("ccpsg", COCO_PSG);
 	device.option_add("dcmodem", COCO_DCMODEM);
 	device.option_add("gmc", COCO_PAK_GMC);
+	device.option_add("ide", COCO_IDE);
 	device.option_add("max", COCO_PAK_MAX);
 	device.option_add("midi", COCO_MIDI);
-	device.option_add("msx2", DRAGON_MSX2);
 	device.option_add("orch90", COCO_ORCH90);
 	device.option_add("ram", COCO_PAK_RAM);
 	device.option_add("rs232", COCO_RS232);
 	device.option_add("ssc", COCO_SSC);
+	device.option_add("ssfm", DRAGON_MSX2);
 	device.option_add("stecomp", COCO_STEREO_COMPOSER);
 	device.option_add("sym12", COCO_SYM12);
 	device.option_add("wpk", COCO_WPK);
@@ -735,12 +858,10 @@ void coco_cart_add_fdcs(device_slot_interface &device)
 {
 	// FDCs are optional because if they are on a Multi-Pak interface, they must
 	// be on Slot 4
-	device.option_add("cc2hdb1", COCO2_HDB1);
-	device.option_add("cc3hdb1", COCO3_HDB1);
 	device.option_add("cd6809_fdc", CD6809_FDC);
 	device.option_add("cp450_fdc", CP450_FDC);
 	device.option_add("fdc", COCO_FDC);
-	device.option_add("fdcv11", COCO_FDC_V11);
+	device.option_add("scii", COCO_SCII);
 }
 
 
@@ -769,13 +890,13 @@ void dragon_cart_add_basic_devices(device_slot_interface &device)
 	device.option_add("jcbspch", DRAGON_JCBSPCH);
 	device.option_add("max", COCO_PAK_MAX);
 	device.option_add("midi", DRAGON_MIDI);
-	device.option_add("msx2", DRAGON_MSX2);
 	device.option_add("orch90", COCO_ORCH90);
 	device.option_add("pak", COCO_PAK);
 	device.option_add("serial", DRAGON_SERIAL);
 	device.option_add("ram", COCO_PAK_RAM);
 	device.option_add("sprites", DRAGON_SPRITES);
 	device.option_add("ssc", COCO_SSC);
+	device.option_add("ssfm", DRAGON_MSX2);
 	device.option_add("stecomp", COCO_STEREO_COMPOSER);
 	device.option_add("sym12", COCO_SYM12);
 	device.option_add("wpk2p", COCO_WPK2P);
