@@ -32,18 +32,18 @@
 // ----------------------------------
 // Flags for debugging
 
-#define LOG_WARN        (1U<<1)    // Warnings
-#define LOG_CONFIG      (1U<<2)    // Configuration
-#define LOG_EPROM       (1U<<3)    // Access to EPROM
-#define LOG_CONTR       (1U<<4)    // Access to controller
-#define LOG_RAM         (1U<<5)    // Access to SRAM
-#define LOG_READY       (1U<<6)    // READY line
-#define LOG_SIGNALS     (1U<<7)    // IRQ and HLD lines
-#define LOG_DRQ         (1U<<8)    // DRQ line (too noisy in SIGNALS)
-#define LOG_DRIVE       (1U<<9)    // Drive operations
-#define LOG_CRU         (1U<<10)   // CRU operations
+#define LOG_WARN        (1U << 1)    // Warnings
+#define LOG_CONFIG      (1U << 2)    // Configuration
+#define LOG_EPROM       (1U << 3)    // Access to EPROM
+#define LOG_CONTR       (1U << 4)    // Access to controller
+#define LOG_RAM         (1U << 5)    // Access to SRAM
+#define LOG_READY       (1U << 6)    // READY line
+#define LOG_SIGNALS     (1U << 7)    // IRQ and HLD lines
+#define LOG_DRQ         (1U << 8)    // DRQ line (too noisy in SIGNALS)
+#define LOG_DRIVE       (1U << 9)    // Drive operations
+#define LOG_CRU         (1U << 10)   // CRU operations
 
-#define VERBOSE ( LOG_GENERAL | LOG_WARN | LOG_CONFIG )
+#define VERBOSE (LOG_GENERAL | LOG_WARN | LOG_CONFIG)
 #include "logmacro.h"
 
 #define CCDCC_TAG "ti99_ccdcc"
@@ -59,14 +59,14 @@
 
 #define BUFFER "ram"
 
-DEFINE_DEVICE_TYPE_NS(TI99_CCDCC, bus::ti99::peb, corcomp_dcc_device, CCDCC_TAG, "CorComp Disk Controller Card")
-DEFINE_DEVICE_TYPE_NS(TI99_CCFDC, bus::ti99::peb, corcomp_fdca_device, CCFDC_TAG, "CorComp Floppy Disk Controller Card Rev A")
+DEFINE_DEVICE_TYPE(TI99_CCDCC, bus::ti99::peb::corcomp_dcc_device, CCDCC_TAG, "CorComp Disk Controller Card")
+DEFINE_DEVICE_TYPE(TI99_CCFDC, bus::ti99::peb::corcomp_fdca_device, CCFDC_TAG, "CorComp Floppy Disk Controller Card Rev A")
 
-DEFINE_DEVICE_TYPE_NS(CCDCC_PALU2, bus::ti99::peb, ccdcc_palu2_device, CCDCC_PALU2_TAG, "CorComp DCC PAL u2")
-DEFINE_DEVICE_TYPE_NS(CCDCC_PALU1, bus::ti99::peb, ccdcc_palu1_device, CCDCC_PALU1_TAG, "CorComp DCC PAL u1")
+DEFINE_DEVICE_TYPE(CCDCC_PALU2, bus::ti99::peb::ccdcc_palu2_device, CCDCC_PALU2_TAG, "CorComp DCC PAL u2")
+DEFINE_DEVICE_TYPE(CCDCC_PALU1, bus::ti99::peb::ccdcc_palu1_device, CCDCC_PALU1_TAG, "CorComp DCC PAL u1")
 
-DEFINE_DEVICE_TYPE_NS(CCFDC_PALU12, bus::ti99::peb, ccfdc_palu12_device, CCFDC_PALU12_TAG, "CorComp FDC PAL u12")
-DEFINE_DEVICE_TYPE_NS(CCFDC_PALU6, bus::ti99::peb, ccfdc_palu6_device, CCFDC_PALU6_TAG, "CorComp FDC PAL u6")
+DEFINE_DEVICE_TYPE(CCFDC_PALU12, bus::ti99::peb::ccfdc_palu12_device, CCFDC_PALU12_TAG, "CorComp FDC PAL u12")
+DEFINE_DEVICE_TYPE(CCFDC_PALU6, bus::ti99::peb::ccfdc_palu6_device, CCFDC_PALU6_TAG, "CorComp FDC PAL u6")
 
 namespace bus::ti99::peb {
 
@@ -102,7 +102,7 @@ void corcomp_fdc_device::setaddress_dbin(offs_t offset, int state)
 /*
     Provides the current address to the PALs.
 */
-uint16_t corcomp_fdc_device::get_address()
+offs_t corcomp_fdc_device::get_address()
 {
 	return m_address;
 }
@@ -133,7 +133,7 @@ bool corcomp_fdc_device::write_access()
 */
 void corcomp_fdc_device::debug_read(offs_t offset, uint8_t* value)
 {
-	uint16_t saveaddress = m_address;
+	offs_t saveaddress = m_address;
 
 	m_address = offset;
 	*value = 0x00;
@@ -147,7 +147,7 @@ void corcomp_fdc_device::debug_read(offs_t offset, uint8_t* value)
 	if (m_ctrlpal->selectdsr())
 	{
 		// EPROM selected
-		uint16_t base = m_banksel? 0x2000 : 0;
+		offs_t base = m_banksel? 0x2000 : 0;
 		uint8_t* rom = &m_dsrrom[base | (m_address & 0x1fff)];
 		*value = *rom;
 	}
@@ -156,7 +156,7 @@ void corcomp_fdc_device::debug_read(offs_t offset, uint8_t* value)
 
 void corcomp_fdc_device::debug_write(offs_t offset, uint8_t data)
 {
-	uint16_t saveaddress = m_address;
+	offs_t saveaddress = m_address;
 	m_address = offset;
 	if (m_ctrlpal->selectram())
 	{
@@ -219,7 +219,7 @@ void corcomp_fdc_device::readz(offs_t offset, uint8_t *value)
 	if (m_ctrlpal->selectdsr())
 	{
 		// EPROM selected
-		uint16_t base = m_banksel? 0x2000 : 0;
+		offs_t base = m_banksel? 0x2000 : 0;
 		uint8_t* rom = &m_dsrrom[base | (m_address & 0x1fff)];
 		*value = *rom;
 
