@@ -33,7 +33,6 @@
 #include "i82586.h"
 #include "hashing.h"
 
-#define LOG_GENERAL (1U << 0)
 #define LOG_FRAMES  (1U << 1)
 #define LOG_FILTER  (1U << 2)
 #define LOG_CONFIG  (1U << 3)
@@ -111,7 +110,7 @@ CFG_PARAMS[] =
 i82586_base_device::i82586_base_device(const machine_config &mconfig, device_type type, const char *tag, device_t *owner, uint32_t clock, endianness_t endian, u8 datawidth, u8 addrwidth)
 	: device_t(mconfig, type, tag, owner, clock)
 	, device_memory_interface(mconfig, *this)
-	, device_network_interface(mconfig, *this, 10.0f)
+	, device_network_interface(mconfig, *this, 10)
 	, m_space_config("shared", endian, datawidth, addrwidth)
 	, m_out_irq(*this)
 	, m_cx(false)
@@ -120,6 +119,7 @@ i82586_base_device::i82586_base_device(const machine_config &mconfig, device_typ
 	, m_rnr(false)
 	, m_initialised(false)
 	, m_reset(false)
+	, m_irq(false)
 	, m_irq_assert(1)
 	, m_cu_state(CU_IDLE)
 	, m_ru_state(RU_IDLE)
@@ -162,10 +162,9 @@ void i82586_base_device::device_start()
 {
 	m_space = &space(0);
 
-	m_out_irq.resolve();
+	m_out_irq.resolve_safe();
 
-	m_cu_timer = timer_alloc(CU_TIMER);
-	m_cu_timer->enable(false);
+	m_cu_timer = timer_alloc(FUNC(i82586_base_device::cu_execute), this);
 
 	save_item(NAME(m_cx));
 	save_item(NAME(m_fr));
@@ -173,6 +172,7 @@ void i82586_base_device::device_start()
 	save_item(NAME(m_rnr));
 	save_item(NAME(m_initialised));
 	save_item(NAME(m_reset));
+	save_item(NAME(m_irq));
 
 	save_item(NAME(m_cu_state));
 	save_item(NAME(m_ru_state));
@@ -189,7 +189,7 @@ void i82586_base_device::device_start()
 
 void i82586_base_device::device_reset()
 {
-	m_cu_timer->enable(false);
+	m_cu_timer->reset();
 
 	m_cx = false;
 	m_fr = false;
@@ -202,16 +202,8 @@ void i82586_base_device::device_reset()
 
 	m_scp_address = SCP_ADDRESS;
 	m_mac_multi = 0;
-}
 
-void i82586_base_device::device_timer(emu_timer &timer, device_timer_id id, int param, void *ptr)
-{
-	switch (id)
-	{
-		case CU_TIMER:
-			cu_execute();
-			break;
-	}
+	set_irq(false);
 }
 
 device_memory_interface::space_config_vector i82586_base_device::memory_space_config() const
@@ -337,12 +329,12 @@ void i82586_base_device::process_scb()
 
 	case CUC_RESUME:
 		m_cu_state = CU_ACTIVE;
-		m_cu_timer->enable(true);
+		m_cu_timer->adjust(attotime::zero);
 		break;
 
 	case CUC_SUSPEND:
 		m_cu_state = CU_SUSPENDED;
-		m_cu_timer->enable(false);
+		m_cu_timer->reset();
 		m_cna = true;
 		break;
 
@@ -407,7 +399,7 @@ void i82586_base_device::update_scb()
 	set_irq(m_cx || m_fr || m_cna || m_rnr);
 }
 
-void i82586_base_device::cu_execute()
+TIMER_CALLBACK_MEMBER(i82586_base_device::cu_execute)
 {
 	// fetch the command block command/status
 	const u32 cb_cs = m_space->read_dword(m_cba);
@@ -501,7 +493,7 @@ void i82586_base_device::cu_complete(const u16 status)
 	if (cb_cs & CB_S)
 	{
 		m_cu_state = CU_SUSPENDED;
-		m_cu_timer->enable(false);
+		m_cu_timer->reset();
 		m_cna = true;
 	}
 
@@ -567,9 +559,16 @@ void i82586_base_device::set_irq(bool irq)
 	{
 		LOG("irq asserted\n");
 
+		// ensure an edge is generated if interrupt already asserted
+		if (m_irq)
+			m_out_irq(!m_irq_assert);
+
 		m_out_irq(m_irq_assert);
-		m_out_irq(!m_irq_assert);
 	}
+	else if (m_irq)
+		m_out_irq(!m_irq_assert);
+
+	m_irq = irq;
 }
 
 u32 i82586_base_device::compute_crc(u8 *buf, int length, bool crc16)
