@@ -17,7 +17,6 @@
     DEBUGGING
 ***************************************************************************/
 
-#define LOG_GENERAL (1U << 0)
 #define LOG_PACKETS (1U << 1)
 #define LOG_TX      (1U << 2)
 #define LOG_RX      (1U << 3)
@@ -62,7 +61,7 @@ smc91c96_device::smc91c96_device(const machine_config &mconfig, const char *tag,
 
 smc91c9x_device::smc91c9x_device(const machine_config &mconfig, device_type type, const char *tag, device_t *owner, uint32_t clock, dev_type device_type)
 	: device_t(mconfig, type, tag, owner, clock)
-	, device_network_interface(mconfig, *this, 10.0f)
+	, device_network_interface(mconfig, *this, 10)
 	, m_device_type(device_type)
 	, m_num_ebuf(16)
 	, m_irq_handler(*this)
@@ -82,7 +81,7 @@ void smc91c9x_device::device_start()
 	m_buffer = std::make_unique<u8[]>(ETHER_BUFFER_SIZE * m_num_ebuf);
 
 	// TX timer
-	m_tx_poll = machine().scheduler().timer_alloc(timer_expired_delegate(FUNC(smc91c9x_device::tx_poll), this));
+	m_tx_poll = timer_alloc(FUNC(smc91c9x_device::tx_poll), this);
 
 	m_irq_handler.resolve_safe();
 
@@ -153,7 +152,7 @@ void smc91c9x_device::device_reset()
 	m_rx_active = 0;
 	m_tx_retry_count = 0;
 
-	m_reg[B0_TCR]          = 0x0000;   m_regmask[B0_TCR]          = 0x3d87;
+	m_reg[B0_TCR]          = 0x0000;   m_regmask[B0_TCR]          = 0xbd87;
 	m_reg[B0_EPH_STATUS]   = 0x4000;   m_regmask[B0_EPH_STATUS]   = 0x0000;
 	m_reg[B0_RCR]          = 0x0000;   m_regmask[B0_RCR]          = 0xc307;
 	m_reg[B0_COUNTER]      = 0x0000;   m_regmask[B0_COUNTER]      = 0x0000;
@@ -406,7 +405,7 @@ int smc91c9x_device::recv_start_cb(u8 *buf, int length)
 	}
 
 	// Check for active transmission
-	if (m_tx_active)
+	if (m_tx_active && !(m_reg[B0_TCR] & FDSE))
 	{
 		// TODO: Update collision counters
 		LOGMASKED(LOG_RX, "transmit active COLLISION, rx packet length %d discarded\n", length);
@@ -527,7 +526,7 @@ void smc91c9x_device::recv_complete_cb(int result)
 TIMER_CALLBACK_MEMBER(smc91c9x_device::tx_poll)
 {
 	// Check for active RX and delay if necessary
-	if (m_rx_active)
+	if (m_rx_active && !(m_reg[B0_TCR] & FDSE))
 	{
 		// TODO: Implement correct CSMA/CD algorithm
 		m_tx_poll->adjust(attotime::from_usec(40));
@@ -659,11 +658,19 @@ void smc91c9x_device::send_complete_cb(int result)
 		m_reg[B2_INTERRUPT] |= EINT_EPH;
 	}
 
-	// Update status in the transmit word
-	*(u16*)&tx_buffer[0] = m_reg[B0_EPH_STATUS];
+	if (m_reg[B1_CONTROL] & AUTO_RELEASE)
+	{
+		alloc_release(packet_num);
+	}
+	else
+	{
+		// Update status in the transmit word
+		*(u16*)&tx_buffer[0] = m_reg[B0_EPH_STATUS];
+
+		// Push the packet number onto the tx completion fifo
+		push_completed_tx(packet_num);
+	}
 
-	// Push the packet number onto the tx completion fifo
-	push_completed_tx(packet_num);
 
 	update_ethernet_irq();
 
